package org.erif.util;

import javax.servlet.http.*;

/** a grab bag of html niftiness.  mainly used for clean(), newlineToBR(), newlineToLI(), and makeSelectList() */
public class HTMLUtil {

  private static final int MULTI_SELECT_SIZE = 5;

  /**
   * returns remote ip as String from HTTPServletRequest
   */
  public static String getRemoteAddr(HttpServletRequest req) {
    return req.getRemoteAddr();
  }

  /**
   * returns referring page as String from HTTPServletRequest
   */
  public static String getReferrer(HttpServletRequest req) {
    return req.getHeader("Referer");
  }
   

  /**
   * removes open brackets from text, neutralizing any HTML present
   * @param text text to be cleaned
   * @return cleaned text
   */
  public static String clean(String text) {
    if (text == null) return null;
    text = TextUtil.replaceString("&","&amp;", text);
    return TextUtil.replaceString("<", "&lt;", text);
  }

  public static String encode(String text) {
    if (text == null) return null;
    text = clean(text);
    text = TextUtil.replaceString("\\","\\\\",text);
    text = TextUtil.replaceString("\"","&quot;",text);
    return text;
  }

  /**
   * replaces newlines with &lt;BR&gt;'s
   * @param text text to be displayed
   * @return HTMLized text, ish
   */
  public static String newlineToBR(String text) {
    return TextUtil.replaceString("\n", "<br />", text);
  }

  /**
   * replaces newlines with &lt;BR&gt;'s
   * @param text text to be displayed
   * @return &lt;li&gt; tags wrapped around said items.
   */
  public static String newlineToLI(String text) {
    String retval = "<li>"+TextUtil.replaceString("\n", "</li><li>", text);
    if (retval.endsWith("<li>")) retval = retval.substring(0,retval.length()-4);
    else retval+="</li>";
    return retval;
  }

  /**
   * Provides HTML for a select box containing the items in the parameters;
   * automatically defaults to previously selected statement
   * 
   * @param name name of select box for use in parameters
   * @param hash object array containing key/value pairs
   * @param chosen value to be used as default SELECTED
   * @param prompt display text for unselected list (or null for none)
   * @return HTML string containing select box
   */
  public static String makeSelectList(String name, String[][] hash, String chosen, String prompt) {
    /* Makes a fully-formed HTML select given required parameters */
    StringBuffer retval = new StringBuffer("");
    retval.append("<select name=\"").append(name).append("\">");
    String key;
    String value;
    if (prompt != null)
      retval.append("<option value=\"-1\">").append(prompt).append("</option>");
    for (int i=0; i<hash.length;i++) {
      key = (String)hash[i][0];
      value = (String)hash[i][1];
      retval.append("<option value=\"").append(value).append("\"");
      if (value.equals(chosen))
        retval.append(" selected=\"selected\"");
      retval.append(">").append(key).append("</option>");
    }
    retval.append("</select>");
    return retval.toString();
  }

  /**
   * Provides HTML for a select box containing the items in the parameters;
   * automatically defaults to previously selected statement
   * 
   * @param name name of select box for use in parameters
   * @param hash object array containing key/value pairs
   * @param chosen value to be used as default SELECTED
   * @return HTML string containing select box
   */
  public static String makeSelectList(String name, String[][] hash, String chosen) {
    return makeSelectList(name, hash, chosen, null);
  }

  /**
   * Provides HTML for a select box containing the items in the parameters;
   * automatically defaults to previously selected statement
   * 
   * @param name name of select box for use in parameters
   * @param hash int array containing numeric values (key/value are the 
   *     same)
   * @param chosen value to be used as default SELECTED
   * @return HTML string containing select box
   */
  public static String makeSelectList(String name, int[] hash, int chosen) {
    /* Makes a fully-formed HTML select given required parameters */
    StringBuffer retval = new StringBuffer("");
    retval.append("<select name=\"").append(name).append("\">");
    int value;
    for (int i=0; i<hash.length;i++) {
      value = hash[i];
      retval.append("<option value=\"").append(value).append("\"");
      if (value == chosen)
        retval.append(" selected=\"selected\"");
      retval.append(">").append(value).append("</option>");
    }
    retval.append("</select>");
    return retval.toString();
  }

  /**
   * Provides HTML for a select box containing the items in the parameters;
   * automatically defaults to previously selected statement
   * 
   * @param name name of select box for use in parameters
   * @param hash String array containing values (key/value are the same)
   * @param chosen value to be used as default SELECTED
   * @return HTML string containing select box
   */
  public static String makeSelectList(String name, String[] hash, String chosen) {
    /* Makes a fully-formed HTML select given required parameters */
    StringBuffer retval = new StringBuffer("");
    retval.append("<select name=\"").append(name).append("\">");
    String value;
    for (int i=0; i<hash.length;i++) {
      value = hash[i];
      retval.append("<option value=\"").append(value).append("\"");
      if (value.equals(chosen))
        retval.append(" selected=\"selected\"");
      retval.append(">").append(value).append("</option>");
    }
    retval.append("</select>");
    return retval.toString();
  }

  /**
   * Provides HTML for a multiselect box containing the items in the parameters;
   * automatically defaults to previously selected statement
   * 
   * @param name name of select box for use in parameters
   * @param hash object array containing key/value pairs
   * @param chosen value(s) to be used as default SELECTED
   * @return HTML string containing select box
   */
  public static String makeMultiSelectList(String name, String[][] hash, String[] chosen) {
    /* Makes a fully-formed HTML select given required parameters */
    StringBuffer retval = new StringBuffer("");
    String size = "";
    if (hash.length >= MULTI_SELECT_SIZE)
      size = " size = \"" + MULTI_SELECT_SIZE + "\"";
    retval.append("<select multiple ").append(size).append("name=\"").append(name).append("\">");
    String key;
    String value;
    for (int i=0; i<hash.length;i++) {
      key = (String)hash[i][0];
      value = (String)hash[i][1];
      retval.append("<option value=\"").append(value).append("\"");
      for (int j=0; j<chosen.length; j++) {
        if (value.equals(chosen[j]))
          retval.append(" selected=\"selected\"");
      }
      retval.append(">").append(key).append("</option>");
    }
    retval.append("</select>");
    return retval.toString();
  }

  /**
   * Provides HTML for a multiselect box containing the items in the parameters
   * 
   * @param name name of select box for use in parameters
   * @param hash object array containing key/value pairs
   * @return HTML string containing select box
   */
  public static String makeMultiSelectList(String name, String[][] hash) {
    return makeMultiSelectList(name, hash, new String[0]);
  }
}
