package org.erif.util;

import java.io.*;
import java.util.*;
import java.text.*;
import javax.mail.internet.InternetAddress;

/** this is partially superseded by java2 v1.4, but, eh... */
public class TextUtil {

  /** repeats a given String n times */
  public static String repeat(String s, int n) {
    StringBuffer sb = new StringBuffer(s.length()*n);
    for (int i=0;i<n;i++) {
      sb.append(s);
    }
    return sb.toString();
  }

  /** replaces all occurrences of old with new in the specified text.
   * @param oldStr Text to be replaced
   * @param newStr Text with which to replace
   * @param textStr String containing the text to be replaced
   * @return New string containing the replaced text
   */
  public static String replaceString(String oldstr, String newstr, String text){
    int lastchar=0;
    StringBuffer buf=new StringBuffer();
    while (text.indexOf(oldstr,lastchar) != -1) {
      buf.append(text.substring(lastchar,text.indexOf(oldstr,lastchar)));
      buf.append(newstr);
      lastchar = text.indexOf(oldstr,lastchar)+oldstr.length();
    }
    buf.append(text.substring(lastchar,text.length()));
    return buf.toString();
  }

  /**
   * makes sure an email address is reasonably valid.  Lazy implementation -
   * it tries to make an InternetAddress.  If it fails, it's invalid.  Simple,
   * no?
   * @param text Potential address to be validated
   * @return Whether the address is valid or not
   */
// TODO
// /^\w+((-\w+)|(\.\w+))*\@[A-Za-z0-9]+((\.|-)[A-Za-z0-9]+)*\.[A-Za-z0-9]+$/;
  public static boolean isValidEmail(String text) {
    try {
      if (text.indexOf("@") == -1) return false;
      new InternetAddress(text);
      return true;
    } catch (Exception e) {
      return false;
    }
  }

  /** word-wraps a string to 72 characters
   * @param text string to wrap
   * @return string with newlines inserted
   */
  public static String wrapForEmail(String text) { return wrap(text, 72); }

  /** word-wraps a string
   * @param text string to wrap
   * @param cols number of columns to wrap to
   * @return string with newlines inserted
   */
  public static String wrap(String text, int cols) {
    StringCharacterIterator itTxt = new StringCharacterIterator(text);
    StringBuffer bufRet = new StringBuffer();
    int pos = 0; // position on line (column #)
    for(char c = itTxt.first(); c != CharacterIterator.DONE; c = itTxt.next()) {
      pos++;
      // add the character to the output buffer
      bufRet.append(c);
      // reset our column count if we hit a line break
      if (c == '\n') pos = 0;
      if (pos > cols) {
        // We've hit the end of the line.  We need to backtrack...
        while (((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >='0' && c <='9')) && pos > 0) {
          // while c is alphanumeric and we're still on the same line
          bufRet.deleteCharAt(bufRet.length()-1); // remove from output buffer,
          pos--; // go back one char,
          c = itTxt.previous(); // and grab it & evaluate (in while loop)
        }
        /* at this point we're either at the beginning of the line or we found
             a breaking point */
        if (pos > 0) {
          // We found a break point.
          bufRet.append('\n');
          pos = 0;
        } else {
          /* we're at the start of the line.  Oh well, I guess we just chop it
             back where it was... */
          while (pos < (cols + 1)) {
            c = itTxt.next();
            pos++;
            bufRet.append(c);
          }
          bufRet.append('\n');
          pos = 0;
        }
      }
    }
    return bufRet.toString();
  }

  /**
   * Given a String[], returns a comma-delimited list of its values.
   * @param array String[] to convert to value list
   * @return Comma-delimited value list
   */
  public static String makeList(String[] array) {
    if (array == null || array.length == 0) return "";
    StringBuffer retbuf = new StringBuffer();
    for (int i=0; i < array.length - 1; i++)
      retbuf.append(array[i]).append(',');
    retbuf.append(array[array.length - 1]);
    return retbuf.toString();
  }

  /**
   * Given a String[], returns a comma-delimited list of its values.
   * @param array String[] to convert to value list
   * @param quote String to put before and after each item of value list
   * @return Comma-delimited value list
   */
  public static String makeList(String[] array,String quote) {
    if (array == null || array.length == 0) return "";
    StringBuffer retbuf = new StringBuffer();
    for (int i=0; i < array.length - 1; i++)
      retbuf.append(quote).append(array[i]).append(quote).append(',');
    retbuf.append(quote).append(array[array.length - 1]).append(quote);
    return retbuf.toString();
  }

  /**
   * Given a int[], returns a comma-delimited list of its values.
   * @param array int[] to convert to value list
   * @return Comma-delimited value list
   */
  public static String makeList(int[] array) {
    if (array == null || array.length == 0) return "";
    StringBuffer retbuf = new StringBuffer();
    for (int i=0; i < array.length - 1; i++)
      retbuf.append(array[i]).append(',');
    retbuf.append(array[array.length - 1]);
    return retbuf.toString();
  }

  /**
   * Given a long[], returns a comma-delimited list of its values.
   * @param array long[] to convert to value list
   * @return Comma-delimited value list
   */
  public static String makeList(long[] array) {
    if (array == null || array.length == 0) return "";
    StringBuffer retbuf = new StringBuffer();
    for (int i=0; i < array.length - 1; i++)
      retbuf.append(array[i]).append(',');
    retbuf.append(array[array.length - 1]);
    return retbuf.toString();
  }

  /**
   * Given a comma-delimited list of numbers, provides an int array if
   * the list contains only numbers and commas.  Silently drops items
   * that aren't numbers.
   * @param list String containing number list
   * @return int array
   */
  public static int[] listToIntArray(String list) {
    StringTokenizer st = new StringTokenizer(list, ",");
    int elements = st.countTokens();
    int[] retval = new int[elements];
    String token = null;
    for (int i=0; i < elements; i++) {
      token = st.nextToken();
      if (isNumOnly(token))
        retval[i] = Integer.parseInt(token);
    }
    return retval;
  }

  /**
   * Given a comma-delimited list of strings, provides an string array.
   * @param list String containing list
   * @return String array
   */
  public static String[] listToStringArray(String list) {
    StringTokenizer st = new StringTokenizer(list, ",");
    int elements = st.countTokens();
    String[] retval = new String[elements];
    for (int i=0; i < elements; i++)
      retval[i] = st.nextToken().trim();
    return retval;
  }

  /**
   * Checks whether the given string contains only digits
   * @param num String to be tested
   * @return whether string contains digits only 
   */
  public static boolean isNumOnly(String num) {
    String s = num + "";
    int len = s.length();
    char c;
    for (int i = 0; i < len; i++) {
      c = s.charAt(i);
      if (c < '0' || c > '9')
        return false;
    }
    return true;
  }

  /** returns the original string value, converting nulls to blanks
   * 
   * @param s string to be stripped of nulls
   * @return original string value or blank if parameter was null
   */
  public static String fixNull(String s) { return (s==null)?"":s; }

  public static String getEmail(String s) {
    if (s.lastIndexOf('<') == -1 || s.lastIndexOf('>') == -1) return s;
    return s.substring(s.lastIndexOf('<')+1,s.lastIndexOf('>'));
  }
  
  /**
   * transforms text rot13-style, baybee
   * 
   * @param s string to be rotated
   * @return rotated text
   */
  public static String rot13(String s) {
    if (s == null) return s;
    int abyte = 0;
    StringBuffer sb = new StringBuffer();
    int cap = 0;
    for (int pos = 0; pos < s.length(); pos++) {
      abyte = s.charAt(pos);
      cap = (byte)abyte & 32;
      abyte &= ~cap;
      abyte = ((abyte >= 'A') && (abyte <= 'Z') ? ((abyte - 'A' + 13) % 26 + 'A') : abyte) | cap;
      sb.append((char)abyte);
    }
    return sb.toString();
  }

  /** given a hash of form name/value pairs and a "letter", returns 
   * the properly merged body; fields are defined by (|fieldname|); 
   * fields with no corresponding value will be returned blank
   *
   * @param subs fieldname/value pairs
   * @param body the contents of the message, containing (|fieldname|) tags
   * @return final body with merge fields replaced
   */
  public static String populate(Hashtable subs,String body) throws IOException {
    return populate(subs,body,true);
  }
  public static String populate(Hashtable subs,String body,boolean load) throws IOException {
    if (load) body = readFile(body);
    StringBuffer retval = new StringBuffer(body.length()*2);
    int start=0, end=0;
    while ((start = body.indexOf("(|",Math.max(start,end))) != -1) {
      retval.append(body.substring(end,start));
      end=body.indexOf("|)",start);
      start+=2;
      retval.append((String)subs.get(body.substring(start,end)));
      end+=2;
    }
    return retval.append(body.substring(end,body.length())).toString();
  }
  public static String populate(String pattern, String replace, String body) {
    StringBuffer retval = new StringBuffer(body.length()*2);
    int start=0, end=0;
    while ((start = body.indexOf("(|",Math.max(start,end))) != -1) {
      retval.append(body.substring(end,start));
      end=body.indexOf("|)",start);
      start+=2;
      if (body.substring(start,end).equalsIgnoreCase(pattern)) {
        retval.append(replace);
      } else {
        retval.append("(|").append(body.substring(start,end)).append("|)");
      }
      end+=2;
    }
    retval.append(body.substring(end,body.length()));
    return retval.toString();
  }

  /** given a filename, return the contents of the file
   * @param s filename
   */
  public static String readFile(String s) throws IOException {
    BufferedReader br = new BufferedReader(new FileReader(s));
    StringBuffer retval = new StringBuffer();
    String temp;
    while ((temp = br.readLine()) != null) {
      retval.append(temp).append('\n');
    }
    return retval.toString();
  }

  /** given a stream, return the contents of the stream
   * @param s stream
   */
  public static String readStream(InputStream is) throws IOException {
    BufferedReader br = new BufferedReader(new InputStreamReader(is));
    StringBuffer retval = new StringBuffer();
    String temp;
    while ((temp = br.readLine()) != null) {
      retval.append(temp).append('\n');
    }
    return retval.toString();
  }

  public static String makeRandomAlNum(int length) {
    Random rn = new Random();
    byte string[] = new byte[length];
    byte b, hi, lo;
    int n;
    for (int i=0;i<length;i++) {
      switch (n=Math.abs(rn.nextInt() % 3)) {
        case 2: lo=(byte)'A'; hi=(byte)'Z'; break;
        case 1: lo=(byte)'a'; hi=(byte)'z'; break;
        case 0: lo=(byte)'2'; hi=(byte)'9'; break;
        default: lo=hi=(byte)'-';
      }
      n=rn.nextInt() % (hi-lo+1);
      if (n<0) n=-n;
      string[i]=(byte)(lo+n);
    }
    return replaceString("i","j",replaceString("I","J",replaceString("l","L",replaceString("O","o",new String(string)))));
  }

  public static String trim(String s, String trailing, int length) {
    if (s == null) return null;
    if (s.length() > length) {
      return s.substring(0,length)+trailing;
    } else return s;
  }
  /** given a hash of mail merge field strings, returns the properly merged
   * body; mail merge fields defined by (|fieldname|); fields with no
   * corresponding value will be returned blank
   *
   * @param subs keyed by fieldname, with value String containing replacement
   * text
   * @param body the contents of the message, containing (|fieldname|) tags
   * @return final body with merge fields replaced
   */
  public static String build(Hashtable subs,String body) {
    if (body == null) return null;
    StringBuffer retval = new StringBuffer(body.length()*2);
    int start=0, end=0;
    while ((start = body.indexOf("(|",Math.max(start,end))) != -1) {
      retval.append(body.substring(end,start));
      end=body.indexOf("|)",start);
      start+=2;
      if (subs != null)
        retval.append((String)subs.get(body.substring(start,end)));
      end+=2;
    }
    return retval.append(body.substring(end,body.length())).toString();
  }

  /**
   * Takes a string and returns null if it's empty, 
   * else returns the original string
   * @param s string to process
   * @return original string or null
   */
  public static String emptyToNull(String s) { return emptyToNull(s,""); }

  /**
   * Takes a string and returns the empty string if null, 
   * else returns the original string.
   * @param s string to process
   * @return empty string or original
   */
  public static String nullToEmpty(String s) { return nullToEmpty(s,""); }

  /**
   * Takes a string and returns null if it matches the empty string, 
   * else returns the original string
   * @param s string to process
   * @param empty string that constitutes "empty"
   * @return original string or null
   */
  public static String emptyToNull(String s, String empty) {
    if (null == s || s.equals(empty))
      return null;
    return s;
  }

  /**
   * Takes a string and returns the empty string if null, 
   * else returns the original string.  If the empty string is
   * null, you're just a dumbass
   * @param s string to process
   * @param empty string that constitutes "empty"
   * @return empty string or original
   */
  public static String nullToEmpty(String s, String empty) {
    return (null==s)?empty:s;
  }

  /**
   * Pads a string to a given length, padding to the left
   * @param orig original string
   * @param pad padding string (typically a single character)
   * @param length minimum length to pad to
   */
  public static String pad(String orig, String pad, int length) {
    return pad(orig,pad,length,true);
  }

  /**
   * Pads a string to a given length, padding to the left or right
   * @param orig original string
   * @param pad padding string (typically a single character)
   * @param length minimum length to pad to
   * @param front boolean: true == front, false == back
   */
  public static String pad(String orig, String pad, int length, boolean front) {
    while (orig.length() < length) orig=front?pad+orig:orig+pad;
    return orig;
  }

  /**
   * removes all non-numeric characters from a String; originally intended
   * for use with phone numbers that have those pesky -, (, ) characters, and
   * spaces.
   * @param string string to strip
   */
  public static String stripNonNumerics(String string) {
    StringBuffer sb = new StringBuffer(string.length());
    for (int i=0;i<string.length();i++) {
      char c = string.charAt(i);
      if (c >='0' && c <= '9') sb.append(c);
    }
    return sb.toString();
  }

  /**
   * test; can currently call "pad"
   */
  public static void main (String[] args) {
    if (args.length == 0) {
      System.out.println("pad: orig pad length [front]");
    } else if ("pad".equalsIgnoreCase(args[0])) {
      String orig = args[1];
      String pad = args[2];
      int length = Integer.parseInt(args[3]);
      if (args.length == 4) {
        System.out.println("pad(\""+orig+"\", \""+pad+"\", "+length+")");
        System.out.println(pad(orig,pad,length));
      } else {
        boolean front = "true".equalsIgnoreCase(args[4]);
        System.out.println("pad(\""+orig+"\", \""+pad+"\", "+length+", "+front+")");
        System.out.println(pad(orig,pad,length,front));
      }
    } else {
      System.err.println("Unknown method: "+args[0]);
      System.err.println("Call with no arguments for help");
    }

  }

}
